@@ -5,63 +5,82 @@ Default ViewSets for web-api.
 import sys
 import logging
 import traceback
+import typing as _t
 from collections import namedtuple
 from copy import deepcopy
 from django.conf import settings
 from django.core import exceptions as djexcs
 from django.http.response import Http404
 from django.db.models.query import QuerySet
-from django.db import transaction
+from django.db import transaction, models
 from rest_framework.reverse import reverse
 from rest_framework import viewsets as vsets, views as rvs, exceptions, status
 from rest_framework.response import Response as RestResponse
+from rest_framework.request import Request
 from rest_framework.decorators import action
 from rest_framework.schemas import AutoSchema as DRFAutoSchema
 from ..exceptions import VSTUtilsException
 from ..utils import classproperty
+from . import responses
 from .serializers import (
-    ErrorSerializer, ValidationErrorSerializer, OtherErrorsSerializer
+    ErrorSerializer,
+    ValidationErrorSerializer,
+    OtherErrorsSerializer,
+    serializers
 )
 
-_ResponseClass = namedtuple("ResponseData", [
+_ResponseClass: _t.Type[_t.NamedTuple] = namedtuple("ResponseData", [
     "data", "status"
 ])
-default_methods = ['get', 'post', 'put', 'patch', 'delete', 'options', 'head']
-logger = logging.getLogger(settings.VST_PROJECT)
+default_methods: _t.List[_t.Text] = [
+    'get',
+    'post',
+    'put',
+    'patch',
+    'delete',
+    'options',
+    'head'
+]
+logger: logging.Logger = logging.getLogger(settings.VST_PROJECT)
 
 
-def exception_handler(exc, context):
-    traceback_str = traceback.format_exc()
+def exception_handler(exc: Exception, context: _t.Any) -> _t.Optional[RestResponse]:
+    traceback_str: _t.Text = traceback.format_exc()
     default_exc = (exceptions.APIException, djexcs.PermissionDenied)
     serializer_class = ErrorSerializer
-    data = None
-    code = status.HTTP_400_BAD_REQUEST
+    data: _t.Optional[_t.Dict[_t.Text, _t.Any]] = None
+    code: _t.SupportsInt = status.HTTP_400_BAD_REQUEST
+
     if isinstance(exc, djexcs.PermissionDenied):  # pragma: no cover
         data = {"detail": str(exc)}
         code = status.HTTP_403_FORBIDDEN
         logger.debug(traceback_str)
+
     elif isinstance(exc, Http404):
         data = {"detail": getattr(exc, 'msg', str(exc))}
         code = status.HTTP_404_NOT_FOUND
         logger.debug(traceback_str)
+
     elif isinstance(exc, djexcs.ValidationError):  # nocv
         if hasattr(exc, 'error_dict'):
-            errors = dict(exc)
+            errors = dict(exc)  # type: ignore
         elif hasattr(exc, 'error_list'):
             errors = {'other_errors': list(exc)}
         else:
-            errors = {'other_errors': str(exc)}
+            errors = {'other_errors': str(exc)}  # type: ignore
         data = {"detail": errors}
         serializer_class = ValidationErrorSerializer
         logger.debug(traceback_str)
+
     elif isinstance(exc, VSTUtilsException):  # nocv
-        data = {"detail": exc.msg, 'error_type': sys.exc_info()[0].__name__}
+        data = {"detail": exc.msg, 'error_type': sys.exc_info()[0].__name__}  # type: ignore
         code = exc.status
         serializer_class = OtherErrorsSerializer
         logger.info(traceback_str)
+
     elif not isinstance(exc, default_exc) and isinstance(exc, Exception):
         data = {
-            'detail': str(sys.exc_info()[1]), 'error_type': sys.exc_info()[0].__name__
+            'detail': str(sys.exc_info()[1]), 'error_type': sys.exc_info()[0].__name__  # type: ignore
         }
         serializer_class = OtherErrorsSerializer
         logger.debug(traceback_str)
@@ -73,13 +92,13 @@ def exception_handler(exc, context):
         except:  # nocv
             pass
         else:
-            return RestResponse(serializer.data, code)
+            return responses.BaseResponseClass(serializer.data, code)
 
     logger.info(traceback_str)
     default_response = rvs.exception_handler(exc, context)
 
     if isinstance(exc, exceptions.NotAuthenticated):  # nocv
-        default_response["X-Anonymous"] = "true"
+        default_response["X-Anonymous"] = "true"  # type: ignore
 
     return default_response
 
@@ -95,7 +114,7 @@ class Response(_ResponseClass):
 
     @property
     def resp(self):
-        return RestResponse(**self._asdict())
+        return responses.BaseResponseClass(**self._asdict())
 
     @property
     def resp_dict(self):  # nocv
@@ -104,11 +123,14 @@ class Response(_ResponseClass):
 
 class AutoSchema(DRFAutoSchema):
 
-    def get_description(self, path, method):
+    def get_description(self, path: _t.Text, method: _t.Text) -> _t.Text:
         # pylint: disable=simplifiable-if-statement,redefined-outer-name
-        method_name = getattr(self.view, 'action', method.lower())
-        method_obj = getattr(self.view, method_name, None)
-        method_view = getattr(method_obj, '_nested_view', None) if method_obj else None
+        method_name: _t.Text = getattr(self.view, 'action', method.lower())
+        method_obj: _t.Optional[_t.Callable] = getattr(self.view, method_name, None)
+        method_view: _t.Optional[_t.Type[rvs.APIView]] = (
+            getattr(method_obj, '_nested_view', None)
+            if method_obj else None
+        )
 
         if method_obj.__doc__:
             return method_obj.__doc__.strip()
@@ -119,7 +141,7 @@ class AutoSchema(DRFAutoSchema):
         action = path.split('/')[-2]
         submethod = getattr(method_view, action, None)
         if submethod.__doc__:
-            return submethod.__doc__.strip()  # nocv
+            return str(submethod.__doc__).strip()  # nocv
         if method == 'GET' and '{' not in path[:-1].split('/')[-1]:
             action = 'list'
         elif method == 'POST':
@@ -132,33 +154,34 @@ class AutoSchema(DRFAutoSchema):
             action = 'partial_update'
         elif method == 'DELETE':
             action = 'destroy'
-        method_view_obj.action = action
+        method_view_obj.action = action  # type: ignore
         if method_view_obj.schema is None:
             return 'No description'  # nocv
-        return method_view_obj.schema.get_description(path, method)
+        return method_view_obj.schema.get_description(path, method)  # type: ignore
 
 
 class QuerySetMixin(rvs.APIView):
-    '''
+    """
     Instance REST operations.
-    '''
+    """
     __slots__ = ()
-    _queryset = None
-    model = None
+    queryset: _t.Optional[QuerySet]
+    _queryset: _t.Optional[QuerySet] = None
+    model: _t.Optional[_t.Type[models.Model]] = None
 
-    @classproperty
-    def queryset(self):
-        # pylint: disable=method-hidden
+    @classproperty  # type: ignore
+    def queryset(self) -> QuerySet:
+        # pylint: disable=method-hidden,function-redefined
         if self._queryset is not None:
             return getattr(self._queryset, 'cleared', self._queryset.all)()
         qs = self.model.objects.all()
         return getattr(qs, 'cleared', qs.all)()
 
-    @queryset.setter
+    @queryset.setter  # type: ignore
     def queryset(self, value):
         self._queryset = value
 
-    def _base_get_queryset(self):
+    def _base_get_queryset(self) -> QuerySet:
         assert self.queryset is not None, (
             "'%s' should either include a `queryset` attribute, "
             "or override the `get_queryset()` method."
@@ -171,10 +194,10 @@ class QuerySetMixin(rvs.APIView):
             queryset = queryset.all()
         return queryset
 
-    def get_extra_queryset(self):
+    def get_extra_queryset(self) -> _t.Optional[QuerySet]:
         return self.queryset
 
-    def get_queryset(self):
+    def get_queryset(self) -> QuerySet:
         if self.queryset is None:  # nocv
             assert self.model is not None, (
                 "'%s' should either include a `queryset` or `model` attribute,"
@@ -189,14 +212,18 @@ class QuerySetMixin(rvs.APIView):
 
 class GenericViewSet(QuerySetMixin, vsets.GenericViewSet):
     __slots__ = ()
-    _serializer_class_one = None
-    model = None
-    action_serializers = {}
+    serializer_class: _t.Type[serializers.Serializer]
+    _serializer_class_one: _t.Optional[_t.Type[serializers.Serializer]] = None
+    model: _t.Optional[_t.Type[models.Model]] = None
+    action_serializers: _t.Dict[_t.Text, serializers.Serializer] = {}
+    _nested_args: _t.Dict[_t.Text, _t.Any]
+    _nested_view: _t.ClassVar[_t.Union[QuerySetMixin, vsets.GenericViewSet]]
+    nested_detail: bool
 
     def filter_for_filter_backends(self, backend):
         return getattr(backend, 'required', False)
 
-    def filter_queryset(self, queryset):
+    def filter_queryset(self, queryset: QuerySet):
         if hasattr(self, 'nested_name'):
             self.filter_backends = list(filter(
                 self.filter_for_filter_backends,
@@ -231,7 +258,7 @@ class GenericViewSet(QuerySetMixin, vsets.GenericViewSet):
         pass
 
     @classmethod
-    def get_view_methods(cls, detail=False):
+    def get_view_methods(cls, detail=False) -> _t.List[_t.Text]:
         attr_name = ''.join(['__', 'detail' if detail else 'list', 'http_methods', '__'])
         methods = getattr(cls, attr_name, None)
         if methods is not None:
@@ -268,7 +295,7 @@ class CopyMixin(GenericViewSet):
     #: Name of field which will get a prefix.
     copy_field_name = 'name'
     #: List of related names which will be copied to new instance.
-    copy_related = []
+    copy_related: _t.List[_t.Text] = []
 
     def copy_instance(self, instance):
         new_instance = deepcopy(instance)
@@ -286,7 +313,7 @@ class CopyMixin(GenericViewSet):
 
     @action(methods=['post'], detail=True)
     @transaction.atomic()
-    def copy(self, request, **kwargs):
+    def copy(self, request: Request, **kwargs) -> responses.BaseResponseClass:
         # pylint: disable=unused-argument
         '''
         Endpoint which copy instance with deps.
@@ -295,7 +322,7 @@ class CopyMixin(GenericViewSet):
         serializer = self.get_serializer(instance, data=request.data, partial=True)
         serializer.is_valid()
         serializer.save()
-        return Response(serializer.data, status.HTTP_201_CREATED).resp
+        return responses.HTTP_201_CREATED(serializer.data)
 
 
 class ModelViewSet(GenericViewSet, vsets.ModelViewSet):
@@ -318,11 +345,11 @@ class ModelViewSet(GenericViewSet, vsets.ModelViewSet):
     Examples:
         .. sourcecode:: python
 
-            from vstutils.api.base import ModelViewSetSet
+            from vstutils.api.base import ModelViewSet
             from . import serializers as sers
 
 
-            class StageViewSet(ModelViewSetSet):
+            class StageViewSet(ModelViewSet):
                 # This is difference with DRF:
                 # we use model instead of queryset
                 model = sers.models.Stage
@@ -343,17 +370,17 @@ class NonModelsViewSet(GenericViewSet):
     __slots__ = ()
     base_name = None
 
-    def get_queryset(self):
+    def get_queryset(self) -> QuerySet:
         return QuerySet()  # nocv
 
 
 class ListNonModelViewSet(NonModelsViewSet, vsets.mixins.ListModelMixin):
     # pylint: disable=abstract-method
     __slots__ = ()
-    schema = None
+    schema = None  # type: ignore
 
     @property
-    def methods(self):
+    def methods(self) -> _t.List[_t.Text]:
         this_class_dict = ListNonModelViewSet.__dict__
         obj_class_dict = self.__class__.__dict__
         new_methods = list()
@@ -363,16 +390,16 @@ class ListNonModelViewSet(NonModelsViewSet, vsets.mixins.ListModelMixin):
                 new_methods.append(name.replace('_', "-"))
         return new_methods
 
-    def list(self, request, *args, **kwargs):
+    def list(self, request: Request, *args, **kwargs) -> responses.BaseResponseClass:
         routes = {
             method: reverse(f"{self.base_name}-{method}", request=request)
             for method in self.methods
         }
-        return Response(routes, status.HTTP_200_OK).resp
+        return responses.HTTP_200_OK(routes)
 
 
 class ReadOnlyModelViewSet(GenericViewSet, vsets.ReadOnlyModelViewSet):
-    """ Default viewset like vstutils.api.base.ModelViewSetSet for readonly models. """
+    """ Default viewset like vstutils.api.base.ModelViewSet for readonly models. """
     __slots__ = ()
 
 
