@@ -1,16 +1,36 @@
 # pylint: disable=protected-access
+import json
 import typing as _t
 from collections import OrderedDict
 from inspect import getmembers
-from django.db import transaction
+from django.db import transaction, models
 from rest_framework.decorators import action
-from rest_framework import response, status
+from rest_framework import response, request as drf_request, status, views, serializers
 from drf_yasg.utils import swagger_auto_schema
 from . import base
 from ..exceptions import VSTUtilsException
 
 
-def __get_nested_path(name: _t.Text, arg: _t.Text = None, arg_regexp: _t.Text = '[0-9]', empty_arg=True) -> _t.Text:
+MasterViewType = _t.Type[base.GenericViewSet]
+
+
+def ensure_is_object(obj):
+    if isinstance(obj, str):
+        return json.loads(obj)
+    return obj
+
+
+def _create_with_iteration(get_serializer_func, nested_manager, data):
+    serializer = get_serializer_func(data=data)
+    serializer.is_valid(raise_exception=True)
+    return nested_manager.create(**serializer.validated_data)
+
+
+def __get_nested_path(
+        name: _t.Text,
+        arg: _t.Text = None,
+        arg_regexp: _t.Text = '[0-9]',
+        empty_arg: bool = True) -> _t.Text:
     path = name
     if not arg:
         return path
@@ -29,13 +49,18 @@ def __get_nested_subpath(*args, **kwargs) -> _t.Text:
     return path
 
 
-def nested_action(name: _t.Text, arg: _t.Text = None, methods=None, manager_name=None, *args, **kwargs) -> _t.Callable:
+def nested_action(
+        name: _t.Text,
+        arg: _t.Text = None,
+        methods=None,
+        manager_name=None,
+        *args, **kwargs) -> _t.Callable:
     # pylint: disable=too-many-locals
-    list_methods = ['get', 'head', 'options', 'post']
-    detail_methods = ['get', 'head', 'options', 'put', 'patch', 'delete']
-    methods = methods or (detail_methods if arg else list_methods)
-    arg_regexp = kwargs.pop('arg_regexp', '[0-9]')
-    empty_arg = kwargs.pop('empty_arg', True)
+    list_methods: _t.List[_t.Text] = ['get', 'head', 'options', 'post']
+    detail_methods: _t.List[_t.Text] = ['get', 'head', 'options', 'put', 'patch', 'delete']
+    methods = list(methods or (detail_methods if arg else list_methods))
+    arg_regexp: _t.Text = kwargs.pop('arg_regexp', '[0-9]')
+    empty_arg: bool = kwargs.pop('empty_arg', True)
     request_arg = kwargs.pop('request_arg', f'{name}_{arg}')
     request_arg = request_arg if arg else None
     append_arg = kwargs.pop('append_arg', arg)
@@ -47,7 +72,8 @@ def nested_action(name: _t.Text, arg: _t.Text = None, methods=None, manager_name
     _nested_filter_class = kwargs.pop('filter_class', None)
 
     def decorator(func: _t.Callable):
-        def wrapper(view: _t.Type[base.GenericViewSet], request: _t.Type[base.RestResponse], *args, **kwargs):
+        # pylint: disable=used-before-assignment
+        def wrapper(view: NestedViewMixin, request: drf_request.Request, *args, **kwargs):
             # Nested name
             view.nested_name = name
             # Allow append to nested or only create
@@ -65,7 +91,7 @@ def nested_action(name: _t.Text, arg: _t.Text = None, methods=None, manager_name
         kwargs['detail'] = True
         kwargs['url_path'] = path
         kwargs['url_name'] = kwargs.pop('url_name', name)
-        view = action(*args, **kwargs)(wrapper)
+        view: NestedViewMixin = action(*args, **kwargs)(wrapper)  # type: ignore
         view._nested_args = _nested_args
         view._nested_manager = manager_name or name
         view._nested_filter_class = _nested_filter_class
@@ -112,13 +138,13 @@ def subaction(*args, **kwargs):
         if operation_description:
             override_kw['operation_description'] = operation_description
         else:
-            override_kw['operation_description'] = str(func.__doc__ or '').strip()
-        return swagger_auto_schema(**override_kw)(func_object)
+            override_kw['operation_description'] = str(func.__doc__ or '').strip()  # type: ignore
+        return swagger_auto_schema(**override_kw)(func_object)  # type: ignore
 
     return decorator
 
 
-def get_action_name(master_view: _t.Type[base.GenericViewSet], method: _t.Text):
+def get_action_name(master_view: MasterViewType, method: _t.Text = '') -> _t.Text:
     method = method.lower()
     if method == 'post':
         action_name = 'create'
@@ -133,34 +159,54 @@ def get_action_name(master_view: _t.Type[base.GenericViewSet], method: _t.Text):
     elif method == 'delete':
         action_name = 'destroy'
     else:  # nocv
-        action_name = None
+        action_name = None  # type: ignore
 
     return action_name
 
 
 class NestedViewMixin:
     __slots__ = ('action',)
+    get_serializer: _t.Callable
+    check_object_permissions: _t.Callable
+    lookup_field: _t.Text
+    request: drf_request.Request
+    nested_name: _t.Text
+    nested_append_arg: _t.Text
+    nested_allow_append: bool
+    nested_arg: _t.Text
+    nested_id: _t.Union[_t.Text, int]
+    nested_view_object: _t.Optional[models.Model]
+    nested_parent_object: _t.Optional[models.Model]
+    nested_detail: bool
+    _nested_filter_class: _t.Any
+    _nested_args: _t.Dict[_t.Text, _t.Any]
+    _nested_manager: _t.Union[models.QuerySet, _t.Text]
+    nested_manager: _t.Union[models.Manager, models.QuerySet]
+    queryset_filters: _t.List
+    master_view: MasterViewType
+    kwargs: _t.Dict
+    get_view_methods: _t.ClassVar[_t.Callable]
 
     def _check_permission_obj(self, objects: _t.Iterable):
         for obj in objects:
             self.check_object_permissions(self.request, obj)
 
-    def get_queryset(self):
+    def get_queryset(self) -> models.QuerySet:
         qs = self.nested_manager.all()
         for qs_filter in self.queryset_filters:
             if callable(qs_filter):
                 qs = qs_filter(self.nested_parent_object, qs)
         return qs
 
-    def get_nested_action_name(self):
-        return get_action_name(self.master_view, self.request.method)
+    def get_nested_action_name(self) -> _t.Text:
+        return get_action_name(self.master_view, str(self.request.method))
 
     def get_serializer_context(self) -> _t.Dict:
-        context = super().get_serializer_context()
+        context = super().get_serializer_context()  # type: ignore
         return context
 
     def perform_destroy(self, instance):
-        if self.master_view.nested_allow_append:
+        if self.master_view.nested_allow_append:  # type: ignore
             self.nested_manager.remove(instance)
         else:
             instance.delete()
@@ -180,7 +226,7 @@ class NestedViewMixin:
 class NestedWithoutAppendMixin(NestedViewMixin):
     __slots__ = ()
 
-    def create(self, request: _t.Type[base.RestResponse], *args, **kwargs):
+    def create(self, request: drf_request.Request, *args, **kwargs):
         # pylint: disable=unused-argument
         many = isinstance(request.data, (list, tuple))
         return self.perform_create_nested(request.data, self.lookup_field, many)
@@ -189,20 +235,17 @@ class NestedWithoutAppendMixin(NestedViewMixin):
         return request_data if many else [request_data]
 
     def _data_create(self, request_data, nested_append_arg):
-        id_list = []
-        for data in request_data:
-            serializer = self.get_serializer(data=data)
-            serializer.is_valid(raise_exception=True)
-            obj = self.nested_manager.create(**serializer.validated_data)
-            id_list.append(getattr(obj, nested_append_arg))
-
-        return id_list
+        get_serializer_func, nested_manager = self.get_serializer, self.nested_manager
+        return [
+            getattr(_create_with_iteration(get_serializer_func, nested_manager, d), nested_append_arg)
+            for d in request_data
+        ]
 
     def perform_create_nested(self, request_data, nested_append_arg, many) -> base.RestResponse:
         id_list = self._data_create(
             self.prepare_request_data(request_data, many), nested_append_arg
         )
-        qs_filter = {nested_append_arg+'__in': id_list}
+        qs_filter = {nested_append_arg + '__in': id_list}
         queryset = self.get_queryset().filter(**qs_filter)
         if not many:
             queryset = queryset.get()
@@ -214,34 +257,48 @@ class NestedWithoutAppendMixin(NestedViewMixin):
 class NestedWithAppendMixin(NestedWithoutAppendMixin):
     def _data_create(self, request_data, nested_append_arg):
         filter_arg = f'{nested_append_arg}__in'
+        request_data = [ensure_is_object(d) for d in request_data]
         objects = self.get_queryset().model.objects.filter(**{
             filter_arg: map(lambda i: i.get(nested_append_arg), request_data)
         })
         self._check_permission_obj(objects)
         self.nested_manager.add(*objects)
         id_list = list(objects.values_list(nested_append_arg, flat=True))
-        not_created = filter(
-            lambda data: data.get(nested_append_arg, None) not in id_list, request_data
-        )
+        handler = self.get_serializer_class()().get_fields()[nested_append_arg].to_representation
+
+        def is_not_created(data):
+            try:
+                return handler(data.get(nested_append_arg, None)) not in id_list
+            except:
+                return True
+
         id_list += super()._data_create(
-            not_created, nested_append_arg
+            filter(is_not_created, request_data), nested_append_arg
         )
         return id_list
 
 
-def nested_view_function(master_view, view, view_request, *args, **kw) -> _t.Type[base.RestResponse]:
+def nested_view_function(
+        master_view: _t.Union[base.GenericViewSet, NestedViewMixin],
+        view: _t.Type[_t.Union[NestedViewMixin, base.GenericViewSet, views.APIView]],
+        view_request: drf_request.Request,
+        *args, **kw) -> base.RestResponse:
     # pylint: disable=unused-argument,unnecessary-lambda
     nested_sub = kw.get('nested_sub', None)
-    view_obj = view()
+    view_obj: _t.Union[NestedViewMixin, base.GenericViewSet, views.APIView] = view()
     view_obj.request = view_request
-    view_obj.kwargs = view_request.parser_context['kwargs']
-    master_view.nested_view_object = view_obj
-    master_view.nested_detail = view.nested_detail
-    return view_obj.dispatch_route(nested_sub)
+    view_obj.kwargs = view_request.parser_context['kwargs']  # type: ignore
+    master_view.nested_view_object = view_obj  # type: ignore
+    master_view.nested_detail = view_obj.nested_detail  # type: ignore
+    return view_obj.dispatch_route(nested_sub)  # type: ignore
 
 
 class BaseClassDecorator:
     __slots__ = 'name', 'arg', 'request_arg', 'args', 'kwargs'
+    name: _t.Text
+    request_arg: _t.Text
+    args: _t.Tuple
+    kwargs: _t.Dict[str, _t.Any]
 
     def __init__(self, name: _t.Text, arg: _t.Text, *args, **kwargs):
         self.name = name
@@ -275,7 +332,7 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
     :type arg: str
     :param view: -- Nested viewset class.
     :type view:
-        vstutils.api.base.ModelViewSetSet,
+        vstutils.api.base.ModelViewSet,
         vstutils.api.base.HistoryModelViewSet,
         vstutils.api.base.ReadOnlyModelViewSet
     :param allow_append: -- Flag for allowing to append existed instances.
@@ -293,17 +350,17 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
         .. sourcecode:: python
 
             from vstutils.api.decorators import nested_view
-            from vstutils.api.base import ModelViewSetSet
+            from vstutils.api.base import ModelViewSet
             from . import serializers as sers
 
 
-            class StageViewSet(ModelViewSetSet):
+            class StageViewSet(ModelViewSet):
                 model = sers.models.Stage
                 serializer_class = sers.StageSerializer
 
 
             nested_view('stages', 'id', view=StageViewSet)
-            class TaskViewSet(ModelViewSetSet):
+            class TaskViewSet(ModelViewSet):
                 model = sers.models.Task
                 serializer_class = sers.TaskSerializer
 
@@ -324,6 +381,11 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
         'methods',
         'queryset_filters'
     )
+    view: _t.Optional[_t.Type[_t.Union[NestedViewMixin, base.GenericViewSet]]]
+    allowed_subs: _t.List[_t.Text]
+    serializers: _t.Tuple[serializers.Serializer, serializers.Serializer]
+    methods: _t.Optional[_t.List[_t.Text]]
+    queryset_filters: _t.List
 
     filter_subs = ['filter', ]
 
@@ -336,18 +398,23 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
         self.queryset_filters = kwargs.pop('queryset_filters', [])
         super().__init__(name, arg, *args, **kwargs)
         self._subs = self.get_subs()
+
         if self.view is None:
             raise self.NoView()
+
         self.serializers = self.__get_serializers(kwargs)
         self.methods = methods
+
         if self.arg is None:
             self.methods = methods or ['get']
+
         self.kwargs['empty_arg'] = self.kwargs.pop('empty_arg', False)
         self.kwargs['append_arg'] = self.arg
         self.kwargs['request_arg'] = self.request_arg
 
     def __get_serializers(self, kwargs):
         serializer_class = kwargs.pop('serializer_class', self.view.serializer_class)
+
         if 'serializer_class_one' in kwargs:
             serializer_class_one = kwargs.pop('serializer_class_one')
         elif hasattr(self.view, 'serializer_class_one'):
@@ -355,6 +422,7 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
         else:
             # This option is deprecated because all viewsets return `serializer_class_one`
             serializer_class_one = serializer_class  # nocv
+
         return (serializer_class, serializer_class_one)
 
     def _get_subs_from_view(self) -> _t.Sequence:
@@ -368,11 +436,13 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
 
     def get_subs(self) -> _t.Sequence:
         subs = self._get_subs_from_view()
+
         if self.allowed_subs is None:
             return []
         elif self.allowed_subs:
             allowed_subs = set(self.allowed_subs)
-            subs = allowed_subs.intersection(subs)
+            subs = list(allowed_subs.intersection(subs))
+
         return subs
 
     @property
@@ -385,7 +455,7 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
 
     def get_view(self, name: _t.Text, **options):
         # pylint: disable=redefined-outer-name
-        mixin_class = NestedViewMixin
+        mixin_class: _t.Type[NestedViewMixin] = NestedViewMixin
         if hasattr(self.view, 'create'):
             if self.kwargs.get('allow_append', False):
                 mixin_class = NestedWithAppendMixin
@@ -402,17 +472,27 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
 
         manager_name = self.kwargs.get('manager_name', self.name)
 
-        def nested_view(view_obj, request, *args, **kwargs):
+        class NestedView(mixin_class, self.view):  # type: ignore
+            __slots__ = ('nested_detail',)
+            __doc__ = self.view.__doc__
+            format_kwarg = None
+            queryset_filters = self.queryset_filters
+
+        NestedView.__name__ = self.view.__name__  # type: ignore
+        NestedView.nested_detail = detail  # type: ignore
+
+        def nested_view_function_wrapper(view_obj, request, *args, **kwargs):
             kwargs.update(options)
             view_obj.nested_parent_object = view_obj.get_object()
-            nested_allow_append = view_obj.nested_allow_append
             nested_append_arg = view_obj.nested_append_arg
             nested_request_arg = view_obj.nested_arg
             nested_parent_object = view_obj.nested_parent_object
+
             if nested_append_arg:
                 nested_id = getattr(nested_parent_object, nested_append_arg, None)
             else:
                 nested_id = None
+
             if callable(manager_name):
                 nested_manager = manager_name(nested_parent_object)
             else:
@@ -423,18 +503,12 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
                     nested_manager_func = getattr(view_obj, view_manager_function_name)
                     nested_manager = nested_manager_func(nested_parent_object)
 
-            class NestedView(mixin_class, self.view):
-                __slots__ = ('nested_detail',)
-                __doc__ = self.view.__doc__
-                master_view = view_obj
-                lookup_field = nested_append_arg
-                lookup_url_kwarg = nested_request_arg
-                format_kwarg = None
-                queryset_filters = self.queryset_filters
-
             NestedView.__name__ = self.view.__name__
+            NestedView.master_view = view_obj
+            NestedView.lookup_field = nested_append_arg
+            NestedView.lookup_url_kwarg = nested_request_arg
             NestedView.nested_detail = detail
-            NestedView.nested_allow_append = nested_allow_append
+            NestedView.nested_allow_append = view_obj.nested_allow_append
             NestedView.nested_append_arg = nested_append_arg
             NestedView.nested_request_arg = nested_request_arg
             NestedView.nested_parent_object = nested_parent_object
@@ -444,10 +518,10 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
             getattr(view_obj, 'nested_allow_check', lambda *args, **kwargs: None)()
             return nested_view_function(view_obj, NestedView, request, *args, **kwargs)
 
-        nested_view.__name__ = name
-        nested_view.__doc__ = self.view.__doc__
-        nested_view._nested_view = self.view
-        return name, nested_view
+        nested_view_function_wrapper.__name__ = name
+        nested_view_function_wrapper.__doc__ = self.view.__doc__
+        nested_view_function_wrapper._nested_view = self.view  # type: ignore
+        return name, nested_view_function_wrapper
 
     def get_view_type(self, type_name: _t.Text, **options):
         return self.get_view(f'{self.name}_{type_name}', **options)
@@ -476,7 +550,7 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
         allowed_methods = set(self.view.get_view_methods(detail))
         return allowed_methods.intersection(methods)
 
-    def decorated(self, detail):
+    def decorated(self, detail) -> _t.Tuple[_t.Text, _t.Type[_t.Union[NestedViewMixin, base.GenericViewSet]]]:
         name, view = self.get_detail_view() if detail else self.get_list_view()
         kwargs = dict(detail=detail)
         kwargs['url_name'] = f'{self.name}-{"detail" if detail else "list"}'
@@ -485,7 +559,7 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
         if self.methods:
             kwargs['methods'] = self._filter_methods(self.methods, detail=detail)
         else:
-            kwargs['methods'] = self.view.get_view_methods(detail)
+            kwargs['methods'] = self.view.get_view_methods(detail)  # type: ignore
         view_class = self.get_decorator(**kwargs)(view)
         view_class._nested_view = self.view
         view_class._nested_name = self.name
@@ -536,10 +610,10 @@ class nested_view(BaseClassDecorator):  # pylint: disable=invalid-name
         return view_class
 
 
-def cache_method_result(func):
+def cache_method_result(func: _t.Callable) -> _t.Callable:
     """Decorator that caches return value of method based on args and kwargs,
     cache value stored in the object instance"""
-    name = f'__cache_{func.__name__}'
+    name: _t.Text = f'__cache_{func.__name__}'
 
     def wrapper(self, *args, **kwargs):
         result = getattr(self, name, None)
