@@ -8,11 +8,12 @@ Read more in Django REST Framework documentation for
 from __future__ import unicode_literals
 
 import json
-
+import typing as _t
 from django.db import models
 from django.contrib.auth import get_user_model
 from rest_framework import serializers, exceptions
 from . import fields
+from .. import utils
 
 
 User = get_user_model()
@@ -60,12 +61,13 @@ class DataSerializer(EmptySerializer):
     def to_internal_value(self, data):
         return data if isinstance(data, self.allowed_data_types) else self.fail("Unknown type.")
 
-    def to_representation(self, value):  # nocv
-        return (
-            json.loads(value)
-            if not isinstance(value, (dict, list))
-            else value
-        )
+    def to_representation(self, value):
+        if not isinstance(value, (dict, list)):
+            result = json.loads(value)
+            if isinstance(result, dict):
+                result = utils.Dict(result)
+            return result
+        return value
 
 
 class JsonObjectSerializer(DataSerializer):
@@ -83,7 +85,7 @@ class ErrorSerializer(DataSerializer):
 
 
 class ValidationErrorSerializer(ErrorSerializer):
-    detail = serializers.DictField(required=True)
+    detail = serializers.DictField(required=True)  # type: ignore
 
 
 class OtherErrorsSerializer(ErrorSerializer):
@@ -157,13 +159,15 @@ class OneUserSerializer(UserSerializer):
 
     class Meta:
         model = User
-        fields = ('id',
-                  'username',
-                  'is_active',
-                  'is_staff',
-                  'first_name',
-                  'last_name',
-                  'email',)
+        fields: _t.Union[_t.List[_t.Text], _t.Tuple] = (
+            'id',
+            'username',
+            'is_active',
+            'is_staff',
+            'first_name',
+            'last_name',
+            'email',
+        )
         read_only_fields = ('is_superuser',
                             'date_joined',)
 
