@@ -1,3 +1,4 @@
+import typing as _t
 import re
 
 from rest_framework import serializers
@@ -8,7 +9,7 @@ class RegularExpressionValidator:
 
     :raises serializers.ValidationError: in case value does not match regular expression
     '''
-    regexp = None
+    regexp: _t.Optional[_t.Pattern[_t.Text]] = None
 
     def __init__(self, regexp=None):
         '''
@@ -26,6 +27,6 @@ class RegularExpressionValidator:
 
 
 class UrlQueryStringValidator(RegularExpressionValidator):
-    '''Class for validation url query string, for example a=&b=1'''
+    """Class for validation url query string, for example a=&b=1"""
 
-    regexp = re.compile(r'^(&?[\w,.-]+=[\w,.-]*)*$')
+    regexp = re.compile(r'^[^&?=].+=.*$')
