@@ -5,19 +5,12 @@ import json
 import traceback
 import logging
 from subprocess import check_call
+from configparserc.config import ConfigParserC
 from ._base import BaseCommand
-from ...config import ConfigParserC  # pylint: disable=import-error
 
 
 logger = logging.getLogger('vstutils')
 
-UWSGI_EXTRA_ARGS = [
-    'listen',
-    'disable-logging',
-    'log-4xx',
-    'log-5xx',
-]
-
 
 class Command(BaseCommand):
     interactive = True
@@ -44,7 +37,7 @@ class Command(BaseCommand):
         logger.debug(f'Prefix={self.prefix} | Project={project_name}')
 
         config = self.prepare_config()
-        env = dict()
+        env = os.environ.copy()
         env[self._settings('CONFIG_ENV_DATA_NAME')] = config.generate_config_string()
         default_envs = {
             'UWSGI_PROCESSES': 'UWSGI_WORKERS',
@@ -243,10 +236,6 @@ class Command(BaseCommand):
             'pidfile': os.getenv(f'{prefix}_UWSGI_PIDFILE', '/run/web.pid'),
             'daemon': 'false'
         }
-        for uwsgi_param in UWSGI_EXTRA_ARGS:
-            uwsgi_param_value = os.getenv(f'{prefix}_UWSGI_{uwsgi_param.upper()}', '')
-            if uwsgi_param_value != '':
-                config['uwsgi'][uwsgi_param] = uwsgi_param_value
 
         # Set worker settings
         config['rpc']['enable_worker'] = 'false'
