@@ -105,7 +105,7 @@ class Command(BaseCommand):
         # Add arguments to uwsgi cmd list.
         cmd.append('--attach-daemon2')
         run = 'stopsignal=15,reloadsignal=1,'
-        run += f'exec={settings.PYTHON_INTERPRETER} -m celery worker'
+        run += 'exec=celery worker'
         run += options
         cmd.append(run)
 
@@ -114,11 +114,17 @@ class Command(BaseCommand):
     def handle(self, *uwsgi_args, **opts):
         super().handle(*uwsgi_args, **opts)
 
+        # Environment
+        env = os.environ.copy()
+
         # Build default uwsgi-command options.
         cmd = [
             str(opts['script']),
             f'--set-ph=program_name={settings.VST_PROJECT}',
             f'--set-ph=lib_name={settings.VST_PROJECT_LIB}',
+            f'--set-ph=asgi_app={settings.ASGI_APPLICATION}',
+            f'--set-ph=api_path={settings.API_URL}',
+            f'--set-ph=ws={"true" if settings.HAS_CHANNELS else "false"}',
             f'--module={settings.UWSGI_APPLICATION}',
         ]
         #  Setup http addr:port.
@@ -158,8 +164,10 @@ class Command(BaseCommand):
         cmd += ['--ini', '-']
 
         # Check if it is run under virtualenv
-        if sys.prefix != sys.base_prefix:
+        if sys.prefix != '/usr':
             cmd += [f'--virtualenv={sys.prefix}']
+            if sys.prefix not in env["PATH"]:
+                env['PATH'] = f'{sys.prefix}/bin:{env["PATH"]}'
 
         # Get config from env
         read, write = os.pipe()
@@ -169,7 +177,7 @@ class Command(BaseCommand):
         # Run web server
         try:
             self._print('Execute: ' + ' '.join(cmd))
-            proc = subprocess.Popen(cmd, env=os.environ.copy(), stdin=read)
+            proc = subprocess.Popen(cmd, env=env, stdin=read)
             try:
                 wait(proc)
             except BaseException as exc:
