@@ -6,19 +6,19 @@ import random  # noqa: F401
 import string  # noqa: F401
 import os  # noqa: F401
 import uuid
+
+from unittest.mock import patch
 from django.db import transaction
 from django.test import TestCase, override_settings  # noqa: F401
 from django.contrib.auth import get_user_model
 
-try:
-    from mock import patch
-except ImportError:  # nocv
-    from unittest.mock import patch
 from .utils import import_class
 
-
 User = get_user_model()
 
+BulkDataType = _t.Union[_t.List[_t.Dict[_t.Text, _t.Any]], str]
+ApiResultType = _t.Union[BulkDataType, _t.Dict, _t.Sequence]
+
 
 class BaseTestCase(TestCase):
     server_name = 'vstutilstestserver'
@@ -43,18 +43,20 @@ class BaseTestCase(TestCase):
         def __exit__(self, exc_type, exc_val, exc_tb):
             self.testcase.user = self.old_user
 
-    def _create_user(self, is_super_user=True):
-        username = self.random_name()
-        email = username + '@gmail.com'
-        password = username.upper()
+    def _create_user(self, is_super_user=True, **kwargs):
+        username = kwargs.pop('username', self.random_name())
+        email = kwargs.pop('email', username + '@gmail.com')
+        password = kwargs.pop('password', username.upper())
         if is_super_user:
             user = User.objects.create_superuser(username=username,
                                                  password=password,
-                                                 email=email)
+                                                 email=email,
+                                                 **kwargs)
         else:
             user = User.objects.create_user(username=username,
                                             password=password,
-                                            email=email)
+                                            email=email,
+                                            **kwargs)
         user.data = {'username': username, 'password': password}
         return user
 
@@ -184,7 +186,7 @@ class BaseTestCase(TestCase):
     def post_result(self, url, code=None, *args, **kwargs):
         return self.get_result("post", url, code, *args, **kwargs)
 
-    def get_result(self, rtype, url, code=None, *args, **kwargs) -> _t.Union[_t.Dict, _t.Sequence]:
+    def get_result(self, rtype, url, code: int = None, *args, **kwargs) -> ApiResultType:
         '''
         Test request with returning result of request
         :param rtype:  - request type (methods from Client cls): get, post etc
@@ -200,13 +202,15 @@ class BaseTestCase(TestCase):
         if kwargs.get("data", False):
             if isinstance(kwargs["data"], str):
                 kwargs["content_type"] = "application/json"
-        result = self.result(request, url, code=code, *args, **kwargs)
+        kwargs['code'] = code
+        result = self.result(request, url, *args, **kwargs)
         self._logout(client)
         return result
 
     def mass_create(self, url, data, *fields):
         '''
         Mass creation objects in api-abstration
+
         :param url: - url to abstract layer
         :param data: - fields of model
         :params fields: - list of fields to check
@@ -251,16 +255,42 @@ class BaseTestCase(TestCase):
         for key, value in kwargs.items():
             self.assertEqual(result[key], value)
 
-    def get_bulk(self, item, data, type, **kwargs):
-        return dict(type=type, item=item, data=data, **kwargs)
+    def endpoint_call(self, data: BulkDataType = None, method: str = 'get', code: int = 200) -> ApiResultType:
+        """
+        Make request to endpoint and assert response status code if specified
 
-    def get_mod_bulk(self, item, pk, data, mtype, method="POST", **kwargs):
-        return self.get_bulk(
-            item, data, 'mod',
-            pk=pk, data_type=mtype, method=method.upper(), **kwargs
-        )
+        :param data: request data
+        :param method: http request method
+        :param code: http status to assert
+        :return: bulk response
+        """
+
+        if data is not None:
+            data = json.dumps(data)
 
-    def make_bulk(self, data, method_type='post'):
         return self.get_result(
-            method_type, self.get_url('_bulk'), 200, data=json.dumps(data)
+            method,
+            f'/{self._settings("VST_API_URL")}/endpoint/',
+            data=data,
+            code=code
         )
+
+    def bulk(self, data: BulkDataType, code: int = 200) -> ApiResultType:
+        """
+        Make non transactional bulk request and assert status code (default is 200)
+
+        :param data: request data
+        :param code: http status to assert
+        :return: bulk response
+        """
+        return self.endpoint_call(data, method='put', code=code)
+
+    def bulk_transactional(self, data: BulkDataType, code: int = 200) -> ApiResultType:
+        """
+        Make transactional bulk request and assert status code (default is 200)
+
+        :param data: request data
+        :param code: http status to assert
+        :return: bulk response
+        """
+        return self.endpoint_call(data, method='post', code=code)
